/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the examples of the Qt Mobility Components.
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Nokia Corporation and its Subsidiary(-ies) nor
**     the names of its contributors may be used to endorse or promote
**     products derived from this software without specific prior written
**     permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
** $QT_END_LICENSE$
**
****************************************************************************/

#include "compassview.h"
#include "compassfilter.h"

#include <QtGui>
#include <QCompass>

CompassView::CompassView(QWidget *parent)
    : QWidget(parent)
{
    lightSand = QColor(0xFEF1E0);
    darkSand = QColor(0xA47F1A).lighter(230);
    lightWood = QColor(0x3B2E2A).lighter();
    darkWood = QColor(0x3B2E2A);
    compassPointer[0] = QPoint(7, -10);
    compassPointer[1] = QPoint(-7,-10);
    chars << "N" << "E" << "S" << "W";
    azimuth = 0;

    compass = new QCompass(this);
    filter = new CompassFilter(this);
    compass->addFilter(filter);
    compass->start();

    compassAvailable = !QSensor::sensorsForType("QCompass").isEmpty();
}

CompassView::~CompassView()
{

}

void CompassView::resizeEvent(QResizeEvent * /* event */)
{
    totalWidth = qMin(width(), height());
    dialWidth = totalWidth * 0.8;
    textWidth = totalWidth * 0.2;
    center = QPoint(width()/2, height()/2);
    gradient = QRadialGradient(center, (dialWidth /2) + 35);
    gradient.setColorAt(1.0, lightSand);
    gradient.setColorAt(0.9, lightSand);
    gradient.setColorAt(0.0, darkSand);
    compassPointer[2] = QPoint(0, (dialWidth/2) * 0.75);

    font = QFont();
    font.setPixelSize(textWidth * 0.5);
    QFontMetrics metrics(font);
    for (int i = 0; i < 4; ++i) {
        charWidths[i] = metrics.width(chars[i]) / 2;
    }
}

void CompassView::paintEvent(QPaintEvent * /* event */)
{
    QPainter painter(this);

    if (!compassAvailable) {
        font.setPointSize(15);
        painter.drawText(rect(), Qt::AlignCenter,"Compass not available");

        return;
    }

    QPen pen;

    painter.fillRect(rect(), lightWood);

    painter.setRenderHint(QPainter::Antialiasing);

    painter.setBrush(gradient);
    pen.setColor(darkWood);
    pen.setWidth(2);
    painter.setPen(pen);

    painter.drawEllipse(center, dialWidth / 2, dialWidth / 2);

    painter.save();

    painter.translate(center);
    painter.rotate(azimuth);

    painter.setBrush(darkWood);

    pen.setColor(darkWood);
    pen.setWidth(3);
    painter.setPen(pen);

    painter.setFont(font);

    for (int i = 0; i < 4; ++i) {
        painter.drawLine(QPoint(0, dialWidth * 0.4), QPoint(0, dialWidth * 0.49));
        pen.setColor(lightSand);
        painter.setPen(pen);
        painter.drawText(-charWidths[i], -((dialWidth/2) + 3), chars[i]);
        pen.setColor(darkWood);
        painter.setPen(pen);
        painter.rotate(90.0);
    }
    painter.rotate(90.0);

    pen.setWidth(2);
    painter.setPen(pen);

    int currRot = 0;
    for (int i = 0; i < 24; ++i) {
        if (currRot % 90) {
            pen.setWidth(2);
            painter.setPen(pen);
            painter.drawLine(QPoint(0, dialWidth * 0.47), QPoint(0, dialWidth * 0.49));
        }
        currRot += 15;
        painter.rotate(15);
    }

    painter.restore();

    painter.translate(center);

    pen.setColor(darkWood);
    pen.setWidth(3);
    painter.setPen(pen);
    painter.setBrush(lightWood);

    painter.rotate(static_cast<qreal>((static_cast<int>(azimuth + 180) % 360)));
    painter.drawConvexPolygon(compassPointer, 3);
}

